import { isObjectType } from "graphql";
import { getTypeName, requireGraphQLSchema } from "../../utils.js";
const RULE_ID = "require-field-of-type-query-in-mutation-result", rule = {
  meta: {
    type: "suggestion",
    docs: {
      category: "Schema",
      description: "Allow the client in one round-trip not only to call mutation but also to get a wagon of data to update their application.\n> Currently, no errors are reported for result type `union`, `interface` and `scalar`.",
      url: `https://the-guild.dev/graphql/eslint/rules/${RULE_ID}`,
      requiresSchema: !0,
      examples: [
        {
          title: "Incorrect",
          code: (
            /* GraphQL */
            `
            type User { ... }

            type Mutation {
              createUser: User!
            }
          `
          )
        },
        {
          title: "Correct",
          code: (
            /* GraphQL */
            `
            type User { ... }

            type Query { ... }

            type CreateUserPayload {
              user: User!
              query: Query!
            }

            type Mutation {
              createUser: CreateUserPayload!
            }
          `
          )
        }
      ]
    },
    schema: []
  },
  create(context) {
    const schema = requireGraphQLSchema(RULE_ID, context), mutationType = schema.getMutationType(), queryType = schema.getQueryType();
    return !mutationType || !queryType ? {} : {
      [`:matches(ObjectTypeDefinition, ObjectTypeExtension)[name.value=${mutationType.name}] > FieldDefinition > .gqlType Name`](node) {
        const typeName = node.value, graphQLType = schema.getType(typeName);
        if (isObjectType(graphQLType)) {
          const { fields } = graphQLType.astNode;
          fields?.some((field) => getTypeName(field) === queryType.name) || context.report({
            node,
            message: `Mutation result type "${graphQLType.name}" must contain field of type "${queryType.name}"`
          });
        }
      }
    };
  }
};
export {
  rule
};
